import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_options.env import ABCUL
from blue_geo import NAME
from blue_geo.catalog.functions import (
    get_catalog_class,
    get_datacube_class_in_catalog,
    get_list_of_collections,
    get_list_of_datacube_classes,
)
from blue_geo.catalog.classes import list_of_catalogs
from blue_geo.catalog.default import as_list_of_args
from blue_geo.catalog.generic.stac.classes import STACCatalog
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

list_of_tasks = "get|list"


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--what",
    default="",
    type=str,
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--catalog",
    default=list_of_catalogs[0],
    type=str,
    help="|".join(list_of_catalogs),
)
parser.add_argument(
    "--datacube_class",
    type=str,
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = args.task in list_of_tasks
item_name = "item"
if args.task == "get":
    if args.what == "is_STAC":
        catalog_class = get_catalog_class(args.catalog)
        print(int(issubclass(catalog_class, STACCatalog)))
    elif args.what.startswith("url:"):
        catalog_class = get_catalog_class(args.catalog)
        print(
            catalog_class.url.get(
                args.what.split("url:", 1)[1],
                "",
            ),
        )
    elif args.what == "url_args":
        catalog_class = get_catalog_class(args.catalog)
        print("|".join(sorted([item for item in catalog_class.url if item])))
    elif args.what == "list_of_args":
        datacube_class = get_datacube_class_in_catalog(
            args.catalog,
            args.datacube_class,
        )
        print(ABCUL.join(as_list_of_args(datacube_class.query_args)))
    else:
        print(f"unknown-{args.what}")
elif args.task == "list":
    output = []

    if args.what == "catalogs":
        item_name = "catalog"
        output = list_of_catalogs
    elif args.what == "collections":
        item_name = "collection"
        output = get_list_of_collections(catalog_class=args.catalog)
    elif args.what in ["datacubes", "datacube_classes"]:
        item_name = "datacube class"
        output = [
            datacube_class.name
            for datacube_class in get_list_of_datacube_classes(
                catalog_class=args.catalog
            )
        ]
    else:
        success = False

    if args.count != -1:
        output = output[: args.count]

    if success:
        if args.log:
            logger.info(
                "{:,} {}(s): {}".format(
                    len(output),
                    item_name,
                    delim.join(output),
                )
            )
        else:
            print(delim.join(output))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from blueness import module
from blue_options.help.functions import help_main

from blue_geo import NAME
from blue_geo.help.functions import help_functions

NAME = module.name(__file__, NAME)


help_main(NAME, help_functions)



import os
from glob import glob

from blue_objects.env import ABCLI_OBJECT_ROOT
from blue_objects import file, path

from blue_geo import VERSION


def generate_seed() -> str:
    path = file.path(__file__)

    apps_path = os.getenv(
        "BLUE_GEO_QGIS_APPS_PATH",
        os.path.join(path, "console/apps"),
    )

    list_of_files = (
        [
            os.path.join(path, f"{module}.py")
            for module in [
                "dependency",
                "console/log",
                "console/project",
                "console/layer",
                "console/fileio",
                "console/application",
                "console/seed",
                "console/QGIS",
                "console/alias",
            ]
        ]
        + sorted(glob(f"{apps_path}/*.py"))
        + [os.path.join(path, "console/main.py")]
    )

    seed = "; ".join(
        [
            f'ABCLI_OBJECT_ROOT="{ABCLI_OBJECT_ROOT}"',
            f'BLUE_GEO_VERSION="{VERSION}"',
        ]
        + [f'exec(Path("{filename}").read_text())' for filename in list_of_files]
    )

    return seed



from typing import List

from blue_options.terminal import show_usage, xtra

from blue_geo.watch.targets.target_list import TargetList

get_what_list = "catalog|collection|exists|one_liner|query_args"


def help_cat(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@targets cat",
            "<target-name>",
        ],
        "cat <target-name>.",
        mono=mono,
    )


def help_cp(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "-"
    return show_usage(
        [
            "@targets cp|copy",
            f"[{options}]",
            "[..|<object-name-1>]",
            "[.|<object-name-2>]",
        ],
        "copy <object-name-1>/target -> <object-name-2>.",
        mono=mono,
    )


def help_download(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "open,QGIS"
    return show_usage(
        [
            "@targets download",
            f"[{options}]",
        ],
        "download watch targets.",
        {
            "object: $BLUE_GEO_WATCH_TARGET_LIST": "",
        },
        mono=mono,
    )


def help_edit(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@targets edit",
        ],
        "edit watch targets.",
        {
            TargetList.filename(): [],
            "object: $BLUE_GEO_WATCH_TARGET_LIST": "",
        },
        mono=mono,
    )


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim space]",
        "[--including_versions 0]",
        "[--target_name <target>]",
        f"[--what <{get_what_list}>]",
    ]
    return show_usage(
        ["@targets get"] + args,
        "get <target> info.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--catalog <catalog>]",
        "[--collection <collection>]",
        "[--count <count>]",
        "[--delim <space>]",
        "[--including_versions 0]",
    ]
    return show_usage(
        ["@targets list"] + args,
        "list targets.",
        mono=mono,
    )


def help_open(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("~QGIS,", mono=mono),
            "template",
        ]
    )
    return show_usage(
        [
            "@targets open",
            f"[{options}]",
        ],
        "open targets.",
        mono=mono,
    )


def help_publish(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "template"
    return show_usage(
        [
            "@targets publish",
            f"[{options}]",
        ],
        "publish watch targets.",
        mono=mono,
    )


def help_save(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "target=all|<target-name>"
    return show_usage(
        [
            "@targets save",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "save target(s) -> <object-name>.",
        {
            "template: $BLUE_GEO_QGIS_TEMPLATE_WATCH": "",
        },
        mono=mono,
    )


def help_test(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@targets test",
        ],
        "test watch targets.",
        mono=mono,
    )


def help_update_template(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~download,target=all|<target-name>,~upload", mono=mono)
    return show_usage(
        [
            "@targets update_template",
            f"[{options}]",
        ],
        "update target template.",
        mono=mono,
    )


def help_upload(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@targets upload",
        ],
        "upload watch targets.",
        {
            "object: $BLUE_GEO_WATCH_TARGET_LIST": "",
        },
        mono=mono,
    )


help_functions = {
    "cat": help_cat,
    "cp": help_cp,
    "download": help_download,
    "edit": help_edit,
    "get": help_get,
    "list": help_list,
    "open": help_open,
    "publish": help_publish,
    "save": help_save,
    "test": help_test,
    "update_template": help_update_template,
    "upload": help_upload,
}



from typing import Tuple, Dict, List, Union, Callable, Any

from blue_options import string

from blue_geo.logger import logger

raster_suffix = [
    ".jp2",
    ".tif",
    ".tiff",
]


class DatacubeScope:
    # description under blue_geo_datacube_ingest
    special_options = [
        "all",
        "metadata",
        "raster",
        "rgb",
        "rgbx",
    ]

    help = "+".join(
        sorted(special_options) + [f"<{suffix}>" for suffix in raster_suffix]
    )

    def __init__(self, what: str):
        list_of_what = what.split("+")

        self.all = "all" in list_of_what

        self.metadata = "metadata" in list_of_what

        self.rgb = "rgb" in list_of_what
        self.rgbx = "rgbx" in list_of_what

        self.raster = "raster" in list_of_what

        self.contains = [
            item for item in list_of_what if item not in self.special_options
        ]

    def filter(
        self,
        list_of_items: List[Dict[str, Any]],  # {"filename": filename, ["size": size]}
        verbose: bool = False,
        needed_for_rgb: Union[Callable, None] = None,
        is_rgb: Union[Callable, None] = None,
    ) -> List[str]:
        list_of_files: List[str] = []

        for item in list_of_items:
            if self.includes(
                item_filename=item["filename"],
                item_size=item.get("size", -1),
                verbose=verbose,
                needed_for_rgb=needed_for_rgb,
                is_rgb=is_rgb,
            ):
                list_of_files.append(item["filename"])

        return list_of_files

    def includes(
        self,
        item_filename: str,
        item_size: int = -1,
        verbose: bool = False,
        needed_for_rgb: Union[Callable, None] = None,
        is_rgb: Union[Callable, None] = None,
    ) -> bool:
        if self.all:
            return True

        if (
            self.metadata
            and item_size <= 10**6
            and not any(item_filename.endswith(suffix) for suffix in raster_suffix)
        ):
            return True

        if self.rgbx and (needed_for_rgb is not None) and needed_for_rgb(item_filename):
            return True

        if self.rgb and (is_rgb is not None) and is_rgb(item_filename):
            return True

        if self.raster and any(
            item_filename.endswith(suffix) for suffix in raster_suffix
        ):
            return True

        if self.contains and any(
            item_filename.endswith(item) for item in self.contains
        ):
            return True

        if verbose:
            logger.info(
                "skipped {}: {}".format(
                    string.pretty_bytes(item_size),
                    item_filename,
                )
            )

        return False



import math
from qgis.core import *
from qgis.gui import *

# version 2.1.1


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def vanwatch_label(row, webdings, feature, parent):
    """
    Produce label text for a vanwatch mapid.

    vanwatch_label(row)
    """
    row = {
        keyword: value
        for keyword, value in row.items()
        if isinstance(value, int) and value != 0
    }

    symbol = {
        "bicycle": "",
        "car": "",
        "person": "",
    }
    scale = 10

    if webdings:
        output = "".join(
            [
                thing
                for thing in [
                    "".join(
                        math.ceil(row.get(thing, 0) / scale) * [symbol.get(thing, "x")]
                    )
                    for thing in symbol.keys()
                ]
                if thing
            ]
        )
        if not output:
            return output

        side = math.ceil(math.sqrt(len(output)))
        matrix = {0: []}
        j = 0
        for i in range(len(output)):
            matrix[j] += [output[i]]
            if len(matrix[j]) >= side:
                j += 1
                matrix[j] = []
        return "\n".join(["".join(thing) for thing in matrix.values()])

    return ", ".join(
        sorted(["{}:{}".format(keyword, value) for keyword, value in row.items()])
    )



from qgis.core import *
from qgis.gui import *

# version 2.1.1


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def vanwatch_display(object_name, cameras, feature, parent):
    """
    Produce display text for a vanwatch mapid.

    vanwatch_display(
        layer_property(@layer,'name'),
        "cameras"
    )
    """
    object_name = object_name.split(" ")[0]

    url_prefix = "https://kamangir-public.s3.ca-central-1.amazonaws.com"

    image_name_list = [url.split("/")[-1].split(".")[0] for url in cameras.split(",")]

    url_list = [
        "{}/{}/{}-inference.jpg".format(
            url_prefix,
            object_name,
            image_name,
        )
        for image_name in image_name_list
    ]

    image_tag_list = [
        f'<a href="{url}"><img src="{url}" height=100 ></a>' for url in url_list
    ]

    return "\n".join(
        [
            '<table border="1">',
            "    <tr>",
        ]
        + [f"        <td>{image_tag}</td>" for image_tag in image_tag_list]
        + [
            "    </tr>",
            "</table>",
            object_name,
        ]
    )



from typing import Dict, Tuple, List
import copy
import geopandas as gpd
from shapely.geometry import Polygon

from blue_objects import file, objects

from blue_geo.logger import logger


class Target:
    def __init__(
        self,
        name: str = "",
        description: str = "",
        catalog: str = "",
        collection: str = "",
        params: Dict[str, str] = {},
        query_args: Dict[str, str] = {},
        urls: Dict[str, str] = {},
        versions: Dict[str, str] = {},
    ) -> None:
        self.name: str = name
        self.description: str = description

        self.catalog = catalog
        self.collection = collection

        self.params = copy.deepcopy(params)
        self.query_args = copy.deepcopy(query_args)

        self.urls = copy.deepcopy(urls)

        self.versions = copy.deepcopy(versions)

    @classmethod
    def from_dict(
        cls,
        name: str,
        data: Dict,
    ) -> "Target":
        description = data.get("description", "")
        if not description:
            candidate_description = [
                url.split(",", 1)[1]
                for url in data.get("urls", {}).values()
                if "," in url
            ]
            if candidate_description:
                description = candidate_description[0].strip()

        return cls(
            name=name,
            description=description,
            catalog=data.get("catalog", ""),
            collection=data.get("collection", ""),
            params=copy.deepcopy(data.get("params", {})),
            query_args=copy.deepcopy(data.get("query_args", {})),
            urls=copy.deepcopy(data.get("urls", {})),
            versions=copy.deepcopy(data.get("versions", {})),
        )

    def get_version(
        self,
        version: str,
        is_full_name: bool = False,
    ) -> "Target":
        if is_full_name:
            version = version[len(self.name) + 1 :]

        if version not in self.versions:
            return Target()

        version_updates = self.versions.get(version, {})

        params = copy.deepcopy(self.params)
        params.update(version_updates.get("params", {}))

        query_args = copy.deepcopy(self.query_args)
        query_args.update(version_updates.get("query_args", {}))

        return Target(
            name=f"{self.name}-{version}",
            description=self.description,
            catalog=self.catalog,
            collection=self.collection,
            params=params,
            query_args=query_args,
            urls=self.urls,
        )

    @property
    def lat_and_lon(self) -> str:
        lat = self.query_args["lat"]
        lon = self.query_args["lon"]

        return '`lat: {:.04f}"{}`, `lon: {:.04f}"{}`.'.format(
            abs(lat),
            "N" if lat > 0 else "S",
            abs(lon),
            "E" if lon > 0 else "W",
        )

    @classmethod
    def load(cls, object_name: str) -> Tuple[bool, "Target"]:
        success, metadata = file.load_yaml(
            objects.path_of(
                "target/metadata.yaml",
                object_name,
            )
        )

        return success, Target.from_dict(
            metadata.get("name", ""),
            metadata,
        )

    @property
    def one_liner(self) -> str:
        return "{}: {}{} | {}/{} | {} | {}{}".format(
            self.__class__.__name__,
            self.name,
            " - {}".format(self.description) if self.description else "",
            self.catalog,
            self.collection,
            self.query_args_as_str(" | "),
            " | ".join([f"{param}={value}" for param, value in self.params.items()]),
            (
                " | {} version(s): {}".format(
                    len(self.versions),
                    ",".join(self.versions.keys()),
                )
                if self.versions
                else ""
            ),
        )

    @property
    def polygon(self) -> Polygon:
        lat = self.query_args.get("lat", 0)
        lon = self.query_args.get("lon", 0)
        width = self.params.get("width", 0.1)
        height = self.params.get("height", 0.1)

        half_width = width / 2
        half_height = height / 2

        top_left = (lon - half_width, lat + half_height)
        top_right = (lon + half_width, lat + half_height)
        bottom_right = (lon + half_width, lat - half_height)
        bottom_left = (lon - half_width, lat - half_height)

        return Polygon(
            [
                top_left,
                top_right,
                bottom_right,
                bottom_left,
                top_left,
            ]
        )

    def query_args_as_str(self, delim: str = " ") -> str:
        return delim.join(
            [f"--{argument} {value}" for argument, value in self.query_args.items()]
        )

    def save(self, object_name: str) -> bool:
        logger.info(f"saving target: {self.name} -> {object_name}")

        if not file.save_yaml(
            objects.path_of(
                "target/metadata.yaml",
                object_name,
                create=True,
            ),
            self.__dict__,
        ):
            return False

        gdf = gpd.GeoDataFrame(
            {
                "id": ["1"],
                "name": [self.name],
                "description": [self.description],
                "catalog": [self.catalog],
                "collection": [self.collection],
                "geometry": [self.polygon],
            },
            crs="EPSG:4326",
        )

        return file.save_geojson(
            objects.path_of("target/shape.geojson", object_name),
            gdf,
            log=True,
        )

    def urls_as_str(self) -> List[str]:
        return sorted(
            [
                " - [{}]({}){}".format(
                    title,
                    url.split(",", 1)[0],
                    (
                        ": {}".format(
                            self.lat_and_lon
                            if title == "Google Maps"
                            else url.split(",", 1)[1].strip()
                        )
                        if "," in url or title == "Google Maps"
                        else ""
                    ),
                )
                for title, url in self.urls.items()
            ]
        )



from blue_geo import NAME, VERSION, DESCRIPTION, REPO_NAME
from blueness.pypi import setup


setup(
    filename=__file__,
    repo_name=REPO_NAME,
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    packages=[
        NAME,
        f"{NAME}.catalog",
        f"{NAME}.catalog.copernicus",
        f"{NAME}.catalog.copernicus.sentinel_2",
        f"{NAME}.catalog.EarthSearch",
        f"{NAME}.catalog.EarthSearch.sentinel_2_l1c",
        f"{NAME}.catalog.SkyFox",
        f"{NAME}.catalog.SkyFox.Venus",
        f"{NAME}.catalog.firms",
        f"{NAME}.catalog.firms.area",
        f"{NAME}.catalog.generic",
        f"{NAME}.catalog.generic.stac",
        f"{NAME}.catalog.generic.generic",
        f"{NAME}.catalog.ukraine_timemap",
        f"{NAME}.catalog.ukraine_timemap.ukraine_timemap",
        #
        f"{NAME}.doc",
        f"{NAME}.datacube",
        #
        f"{NAME}.help",
        f"{NAME}.help.watch",
        f"{NAME}.help.QGIS",
        #
        f"{NAME}.QGIS",
        f"{NAME}.QGIS.console",
        f"{NAME}.QGIS.console.apps",
        f"{NAME}.QGIS.expressions",
        #
        f"{NAME}.objects",
        f"{NAME}.objects.md",
        #
        f"{NAME}.watch",
        f"{NAME}.watch.algo",
        f"{NAME}.watch.algo.diff",
        f"{NAME}.watch.algo.modality",
        f"{NAME}.watch.targets",
        f"{NAME}.watch.targets.md",
        f"{NAME}.watch.workflow",
    ],
    include_package_data=True,
    package_data={
        NAME: [
            "config.env",
            "sample.env",
            ".abcli/**/*.sh",
            "**/*.md",
        ],
    },
)
